# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: bidirectional.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='bidirectional.proto',
  package='bidirectional',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13\x62idirectional.proto\x12\rbidirectional\"\x1a\n\x07Message\x12\x0f\n\x07message\x18\x01 \x01(\t\"&\n\x13SeverExpStartReturn\x12\x0f\n\x07message\x18\x01 \x01(\t\"$\n\x0fStartExperiment\x12\x11\n\texp_token\x18\x01 \x01(\t\"*\n\x15GetExperimentResponse\x12\x11\n\texp_token\x18\x01 \x01(\t\"O\n\nSDKPayload\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x12\n\nproject_id\x18\x03 \x01(\t\x12\x0e\n\x06\x65xp_id\x18\x04 \x01(\t\"\xb4\x01\n\x12\x45XPAlectioResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x12\n\nproject_id\x18\x03 \x01(\t\x12\x10\n\x08\x63ur_loop\x18\x04 \x01(\t\x12\x0f\n\x07user_id\x18\x05 \x01(\t\x12\x13\n\x0b\x62ucket_name\x18\x06 \x01(\t\x12\x0c\n\x04type\x18\x07 \x01(\t\x12\r\n\x05n_rec\x18\x08 \x01(\x02\x12\x0e\n\x06n_loop\x18\t \x01(\x02\x32\x9f\x02\n\rBidirectional\x12I\n\x11GetServerResponse\x12\x16.bidirectional.Message\x1a\x16.bidirectional.Message\"\x00(\x01\x30\x01\x12\x62\n\x1aGetStartExperimentResponse\x12\x1e.bidirectional.StartExperiment\x1a\x1e.bidirectional.StartExperiment\"\x00(\x01\x30\x01\x12_\n\x0eGetSDKResponse\x12$.bidirectional.GetExperimentResponse\x1a!.bidirectional.EXPAlectioResponse\"\x00(\x01\x30\x01\x62\x06proto3'
)




_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='bidirectional.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='bidirectional.Message.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=64,
)


_SEVEREXPSTARTRETURN = _descriptor.Descriptor(
  name='SeverExpStartReturn',
  full_name='bidirectional.SeverExpStartReturn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='bidirectional.SeverExpStartReturn.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=104,
)


_STARTEXPERIMENT = _descriptor.Descriptor(
  name='StartExperiment',
  full_name='bidirectional.StartExperiment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exp_token', full_name='bidirectional.StartExperiment.exp_token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=142,
)


_GETEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='GetExperimentResponse',
  full_name='bidirectional.GetExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exp_token', full_name='bidirectional.GetExperimentResponse.exp_token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=144,
  serialized_end=186,
)


_SDKPAYLOAD = _descriptor.Descriptor(
  name='SDKPayload',
  full_name='bidirectional.SDKPayload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bidirectional.SDKPayload.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='bidirectional.SDKPayload.user_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_id', full_name='bidirectional.SDKPayload.project_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exp_id', full_name='bidirectional.SDKPayload.exp_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=188,
  serialized_end=267,
)


_EXPALECTIORESPONSE = _descriptor.Descriptor(
  name='EXPAlectioResponse',
  full_name='bidirectional.EXPAlectioResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bidirectional.EXPAlectioResponse.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='bidirectional.EXPAlectioResponse.experiment_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_id', full_name='bidirectional.EXPAlectioResponse.project_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cur_loop', full_name='bidirectional.EXPAlectioResponse.cur_loop', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='bidirectional.EXPAlectioResponse.user_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bucket_name', full_name='bidirectional.EXPAlectioResponse.bucket_name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='bidirectional.EXPAlectioResponse.type', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='n_rec', full_name='bidirectional.EXPAlectioResponse.n_rec', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='n_loop', full_name='bidirectional.EXPAlectioResponse.n_loop', index=8,
      number=9, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=270,
  serialized_end=450,
)

DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.message_types_by_name['SeverExpStartReturn'] = _SEVEREXPSTARTRETURN
DESCRIPTOR.message_types_by_name['StartExperiment'] = _STARTEXPERIMENT
DESCRIPTOR.message_types_by_name['GetExperimentResponse'] = _GETEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['SDKPayload'] = _SDKPAYLOAD
DESCRIPTOR.message_types_by_name['EXPAlectioResponse'] = _EXPALECTIORESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.Message)
  })
_sym_db.RegisterMessage(Message)

SeverExpStartReturn = _reflection.GeneratedProtocolMessageType('SeverExpStartReturn', (_message.Message,), {
  'DESCRIPTOR' : _SEVEREXPSTARTRETURN,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.SeverExpStartReturn)
  })
_sym_db.RegisterMessage(SeverExpStartReturn)

StartExperiment = _reflection.GeneratedProtocolMessageType('StartExperiment', (_message.Message,), {
  'DESCRIPTOR' : _STARTEXPERIMENT,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.StartExperiment)
  })
_sym_db.RegisterMessage(StartExperiment)

GetExperimentResponse = _reflection.GeneratedProtocolMessageType('GetExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETEXPERIMENTRESPONSE,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.GetExperimentResponse)
  })
_sym_db.RegisterMessage(GetExperimentResponse)

SDKPayload = _reflection.GeneratedProtocolMessageType('SDKPayload', (_message.Message,), {
  'DESCRIPTOR' : _SDKPAYLOAD,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.SDKPayload)
  })
_sym_db.RegisterMessage(SDKPayload)

EXPAlectioResponse = _reflection.GeneratedProtocolMessageType('EXPAlectioResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXPALECTIORESPONSE,
  '__module__' : 'bidirectional_pb2'
  # @@protoc_insertion_point(class_scope:bidirectional.EXPAlectioResponse)
  })
_sym_db.RegisterMessage(EXPAlectioResponse)



_BIDIRECTIONAL = _descriptor.ServiceDescriptor(
  name='Bidirectional',
  full_name='bidirectional.Bidirectional',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=453,
  serialized_end=740,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetServerResponse',
    full_name='bidirectional.Bidirectional.GetServerResponse',
    index=0,
    containing_service=None,
    input_type=_MESSAGE,
    output_type=_MESSAGE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetStartExperimentResponse',
    full_name='bidirectional.Bidirectional.GetStartExperimentResponse',
    index=1,
    containing_service=None,
    input_type=_STARTEXPERIMENT,
    output_type=_STARTEXPERIMENT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSDKResponse',
    full_name='bidirectional.Bidirectional.GetSDKResponse',
    index=2,
    containing_service=None,
    input_type=_GETEXPERIMENTRESPONSE,
    output_type=_EXPALECTIORESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_BIDIRECTIONAL)

DESCRIPTOR.services_by_name['Bidirectional'] = _BIDIRECTIONAL

# @@protoc_insertion_point(module_scope)
